package com.hw.admin.system.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSON;
import com.hw.admin.model.domain.WorkPacker;
import com.hw.admin.model.domain.WorkPackerExcel;
import com.hw.admin.model.mapper.WorkPackerMapper;
import com.hw.admin.system.utils.DateExUtils;
import com.hw.admin.system.utils.Sequence;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;

import java.util.ArrayList;
import java.util.List;

// 有个很重要的点 DemoDataListener 不能被spring管理，要每次读取excel都要new,然后里面用到spring可以构造方法传进去
public class WorkPackerListener extends AnalysisEventListener<WorkPackerExcel> {
    private static final Logger LOGGER = LoggerFactory.getLogger(WorkPackerListener.class);
    /**
     * 每隔5条存储数据库，实际使用中可以3000条，然后清理list ，方便内存回收
     */
    private static final int BATCH_COUNT = 3000;
    /**
     * 缓存的数据
     */
    private List<WorkPacker> list = new ArrayList<>(BATCH_COUNT);
    /**
     * 假设这个是一个DAO，当然有业务逻辑这个也可以是一个service。当然如果不用存储这个对象没用。
     */
    private WorkPackerMapper workPackerMapper;

    /**
     * 如果使用了spring,请使用这个构造方法。每次创建Listener的时候需要把spring管理的类传进来
     */
    public WorkPackerListener(WorkPackerMapper workPackerMapper) {
        this.workPackerMapper = workPackerMapper;
    }

    /**
     * 这个每一条数据解析都会来调用
     *
     * @param data    one row value. Is is same as {@link AnalysisContext#readRowHolder()}
     * @param context
     */
    Sequence sequence = new Sequence(2, 3);

    @Override
    public void invoke(WorkPackerExcel data, AnalysisContext context) {
        LOGGER.info("解析到一条数据:{}", JSON.toJSONString(data));
        WorkPacker workPacker = new WorkPacker();
        BeanUtils.copyProperties(data, workPacker);
        workPacker.setId(sequence.nextId());
        String s = DateExUtils.commonDatePathStr();
        workPacker.setInsertTime(s);
        list.add(workPacker);
        // 达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
        if (list.size() >= BATCH_COUNT) {
            saveData();
            // 存储完成清理 list
            list = new ArrayList<>(BATCH_COUNT);
        }
    }

    /**
     * 所有数据解析完成了 都会来调用
     *
     * @param context
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        // 这里也要保存数据，确保最后遗留的数据也存储到数据库
        saveData();
        LOGGER.info("所有数据解析完成！");
    }

    /**
     * 加上存储数据库
     */
    private void saveData() {
        LOGGER.info("{}条数据，开始存储数据库！", list.size());
        workPackerMapper.inserList(list);
        LOGGER.info("存储数据库成功！");
    }
}
